<?php
/**
 * Init
 *
 * @author Jegtheme
 * @since 1.0.0
 * @package JEG_WEATHER
 */

namespace JEG_WEATHER;

/** Exit if accessed directly */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Init
 *
 * @package JEG_WEATHER
 */
class Init {

	/**
	 * Instance for Init
	 *
	 * @var Init
	 */
	private static $instance;

	/**
	 * Hold instance of object
	 *
	 * @var array
	 */
	private $object;

	/**
	 * Init constructor.
	 */
	private function __construct() {
		$this->setup_option();
		$this->setup_hook();
	}

	/**
	 * Load plugin assest
	 */
	public function load_assets() {
		if ( is_admin() ) {
			wp_enqueue_style( 'jeg-weather-admin', JEG_WEATHER_URL . '/assets/css/admin/main.css', [], JEG_WEATHER_VERSION );
		} else {
			wp_enqueue_style( 'jeg-weather', JEG_WEATHER_URL . '/assets/css/style.css', null, JEG_WEATHER_VERSION );
			wp_enqueue_script(
				'jeg-weather',
				JEG_WEATHER_URL . '/assets/js/main.js',
				[ 'jquery' ],
				JEG_WEATHER_VERSION,
				true
			);
			wp_enqueue_style( 'jeg-icon', JEG_WEATHER_URL . '/assets/fonts/jegicon/jegicon.css', null, JEG_WEATHER_VERSION );
		}
	}

	/**
	 * Setup Init option
	 */
	private function setup_option() {
		$this->object['options'] = new Options();
		$this->object['weather'] = Weather::get_instance();
	}

	/**
	 * Setup Init hook
	 */
	private function setup_hook() {
		add_action( 'init', [ $this->object['weather'], 'setup_cookie' ] );
		if ( is_admin() ) {
			add_action( 'admin_enqueue_scripts', [ $this, 'load_assets' ] );
			/** Integration Elementor */
			add_action( 'elementor/editor/after_enqueue_styles', [ $this, 'load_assets' ] );
		} else {
			add_action( 'wp_print_styles', [ $this, 'load_assets' ] );
		}

		/** Integration jeg-element */
		add_filter( 'jeg_register_elements', [ $this, 'register_element' ] );
		add_filter( 'jeg_add_image_size', [ $this, 'register_image_size' ] );
		add_filter( 'jeg_show_default_module_customizer', '__return_false' );

		/** Epic dashboard integration */
		add_filter( 'epic_plugin_list', [ $this, 'epic_plugin_list' ] );
		add_filter( 'epic_parent_menu', [ $this, 'epic_parent_menu' ] );
		add_filter( 'epic_dashboard_welcome_string', [ $this, 'epic_welcome_string' ] );
		add_filter( 'epic_dashboard_about_string', [ $this, 'epic_about_string' ] );

	}

	/**
	 * Override parent menu
	 *
	 * @param $value
	 *
	 * @return mixed
	 */
	public function epic_parent_menu( $value ) {
		$value['page_title'] = esc_html__( 'Jegtheme', 'jeg-weather' );
		$value['menu_title'] = esc_html__( 'Jegtheme', 'jeg-weather' );

		return $value;
	}

	/**
	 * Override welcome string
	 * epic-dashboard lib
	 *
	 * @param $value
	 *
	 * @return string
	 */
	public function epic_welcome_string( $value ) {
		return wp_kses( __( 'Welcome to <strong>Jeg Weather Elements Plugin</strong>', 'jeg-weather' ), wp_kses_allowed_html() );
	}

	/**
	 * Override about string
	 * epic-dashboard lib
	 *
	 * @param $value
	 *
	 * @return string
	 */
	public function epic_about_string( $value ) {
		return wp_kses( __( 'Please activate the plugin license to get official support service and automatic update. Read below for additional information.', 'jeg-weather' ), wp_kses_allowed_html() );
	}

	/**
	 * Singleton page for Init class
	 *
	 * @return Init
	 */
	public static function get_instance() {
		if ( null === static::$instance ) {
			static::$instance = new static();
		}

		return static::$instance;
	}

	/**
	 * Get plugin data
	 *
	 * @param $plugins
	 *
	 * @return array
	 */
	public function epic_plugin_list( $plugins ) {
		$plugins[] = [
			'id'      => JEG_WEATHER_ID,
			'name'    => JEG_WEATHER_NAME,
			'slug'    => JEG_WEATHER,
			'version' => JEG_WEATHER_VERSION,
			'option'  => 'jeg_weather_license',
			'file'    => JEG_WEATHER_FILE,
		];

		return $plugins;
	}

	/**
	 * Register all image size
	 *
	 * @param array $sizes List of image size.
	 *
	 * @return mixed
	 */
	public function register_image_size( $sizes ) {
		$sizes['120x86'] = [ 120, 86, true, 715 ];

		return $sizes;
	}

	/**
	 * Register Element
	 *
	 * @param array $elements list element.
	 *
	 * @return mixed
	 */
	public function register_element( $elements ) {
		foreach ( $this->list_elements() as $item ) {
			$item_key              = 'weather_' . strtolower( $item );
			$namespace             = 'JEG_WEATHER\Elements';
			$elements[ $item_key ] =
				[
					'option'    => $namespace . '\Options\\' . $item_key . '_Option',
					'view'      => $namespace . '\Views\\' . $item_key . '_View',
					'elementor' => $namespace . '\Elementor\\' . $item_key . '_Elementor',
				];
		}

		return $elements;
	}

	/**
	 * List Elements
	 *
	 * @return array
	 */
	public function list_elements() {
		return [
			'Main',
			'Inline',
			'Mini',
		];
	}
}
